@@ -5,6 +5,8 @@
  *
  * Author: Dag-Erling Smørgrav <des@des.no>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -29,15 +31,10 @@
 
 #include "config.h"
 
-#include <execinfo.h>
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 
-#include "vtim.h"
-#include "vcs.h"
-
 #include "cache_varnishd.h"
 #include "cache_transport.h"
 
@@ -47,46 +44,27 @@
 
 #include "storage/storage.h"
 #include "vcli_serve.h"
+#include "vtim.h"
+#include "vbt.h"
+#include "vcs.h"
+#include "vtcp.h"
+#include "vsa.h"
 
 /*
- * The panic string is constructed in memory, then copied to the
+ * The panic string is constructed in a VSB, then copied to the
  * shared memory.
  *
  * It can be extracted post-mortem from a core dump using gdb:
  *
- * (gdb) printf "%s", panicstr
+ * (gdb) p *(char **)((char *)pan_vsb+8)
+ *
  */
 
 static struct vsb pan_vsb_storage, *pan_vsb;
 static pthread_mutex_t panicstr_mtx;
 
 static void pan_sess(struct vsb *, const struct sess *);
-
-/*--------------------------------------------------------------------*/
-
-const char *
-body_status_2str(enum body_status e)
-{
-	switch (e) {
-#define BODYSTATUS(U,l)	case BS_##U: return (#l);
-#include "tbl/body_status.h"
-	default:
-		return ("?");
-	}
-}
-
-/*--------------------------------------------------------------------*/
-
-static const char *
-reqbody_status_2str(enum req_body_state_e e)
-{
-	switch (e) {
-#define REQ_BODY(U) case REQ_BODY_##U: return("R_BODY_" #U);
-#include "tbl/req_body.h"
-	default:
-		return("?");
-	}
-}
+static void pan_req(struct vsb *, const struct req *);
 
 /*--------------------------------------------------------------------*/
 
@@ -103,17 +81,27 @@ boc_state_2str(enum boc_state_e e)
 
 /*--------------------------------------------------------------------*/
 
-const char *
-sess_close_2str(enum sess_close sc, int want_desc)
+static void
+pan_stream_close(struct vsb *vsb, stream_close_t sc)
 {
-	switch (sc) {
-	case SC_NULL:		return(want_desc ? "(null)": "NULL");
-#define SESS_CLOSE(nm, s, err, desc)			\
-	case SC_##nm: return(want_desc ? desc : #nm);
-#include "tbl/sess_close.h"
 
-	default:		return(want_desc ? "(invalid)" : "INVALID");
-	}
+	if (sc != NULL && sc->magic == STREAM_CLOSE_MAGIC)
+		VSB_printf(vsb, "%s(%s)", sc->name, sc->desc);
+	else
+		VSB_printf(vsb, "%p", sc);
+}
+
+/*--------------------------------------------------------------------*/
+
+static void
+pan_storage(struct vsb *vsb, const char *n, const struct stevedore *stv)
+{
+
+	if (stv != NULL && stv->magic == STEVEDORE_MAGIC)
+		VSB_printf(vsb, "%s = %s(%s,%s),\n",
+		    n, stv->name, stv->ident, stv->vclname);
+	else
+		VSB_printf(vsb, "%s = %p,\n", n, stv);
 }
 
 /*--------------------------------------------------------------------*/
@@ -123,73 +111,65 @@ static const void *already_list[N_ALREADY];
 static int already_idx;
 
 int
-PAN_already(struct vsb *vsb, const void *ptr)
+PAN__DumpStruct(struct vsb *vsb, int block, int track, const void *ptr,
+    const char *smagic, unsigned magic, const char *fmt, ...)
 {
+	va_list ap;
+	const unsigned *uptr;
 	int i;
 
+	AN(vsb);
+	va_start(ap, fmt);
+	VSB_vprintf(vsb, fmt, ap);
+	va_end(ap);
 	if (ptr == NULL) {
-		VSB_printf(vsb, "},\n");
-		return (1);
+		VSB_cat(vsb, " = NULL\n");
+		return (-1);
 	}
-	for (i = 0; i < already_idx; i++) {
-		if (already_list[i] == ptr) {
-			VSB_printf(vsb, "  [Already dumped, see above]\n");
-			VSB_printf(vsb, "},\n");
-			return (1);
+	VSB_printf(vsb, " = %p {", ptr);
+	if (block)
+		VSB_putc(vsb, '\n');
+	if (track) {
+		for (i = 0; i < already_idx; i++) {
+			if (already_list[i] == ptr) {
+				VSB_cat(vsb, "  [Already dumped, see above]");
+				if (block)
+					VSB_putc(vsb, '\n');
+				VSB_cat(vsb, "},\n");
+				return (-2);
+			}
 		}
+		if (already_idx < N_ALREADY)
+			already_list[already_idx++] = ptr;
 	}
-	if (already_idx < N_ALREADY)
-		already_list[already_idx++] = ptr;
+	uptr = ptr;
+	if (*uptr != magic) {
+		VSB_printf(vsb, "  .magic = 0x%08x", *uptr);
+		VSB_printf(vsb, " EXPECTED: %s=0x%08x", smagic, magic);
+		if (block)
+			VSB_putc(vsb, '\n');
+		VSB_cat(vsb, "}\n");
+		return (-3);
+	}
+	if (block)
+		VSB_indent(vsb, 2);
 	return (0);
 }
 
 /*--------------------------------------------------------------------*/
 
-static void
-pan_ws(struct vsb *vsb, const struct ws *ws)
-{
-
-	VSB_printf(vsb, "ws = %p {\n", ws);
-	if (PAN_already(vsb, ws))
-		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, ws, WS_MAGIC);
-	if (ws->id[0] != '\0' && (!(ws->id[0] & 0x20)))
-		VSB_printf(vsb, "OVERFLOWED ");
-	VSB_printf(vsb, "id = \"%s\",\n", ws->id);
-	VSB_printf(vsb, "{s, f, r, e} = {%p", ws->s);
-	if (ws->f >= ws->s)
-		VSB_printf(vsb, ", +%ld", (long) (ws->f - ws->s));
-	else
-		VSB_printf(vsb, ", %p", ws->f);
-	if (ws->r >= ws->s)
-		VSB_printf(vsb, ", +%ld", (long) (ws->r - ws->s));
-	else
-		VSB_printf(vsb, ", %p", ws->r);
-	if (ws->e >= ws->s)
-		VSB_printf(vsb, ", +%ld", (long) (ws->e - ws->s));
-	else
-		VSB_printf(vsb, ", %p", ws->e);
-	VSB_printf(vsb, "},\n");
-	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
-}
-
-/*--------------------------------------------------------------------*/
-
 static void
 pan_htc(struct vsb *vsb, const struct http_conn *htc)
 {
 
-	VSB_printf(vsb, "http_conn = %p {\n", htc);
-	if (PAN_already(vsb, htc))
+	if (PAN_dump_struct(vsb, htc, HTTP_CONN_MAGIC, "http_conn"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, htc, HTTP_CONN_MAGIC);
 	if (htc->rfd != NULL)
 		VSB_printf(vsb, "fd = %d (@%p),\n", *htc->rfd, htc->rfd);
-	VSB_printf(vsb, "doclose = %s,\n", sess_close_2str(htc->doclose, 0));
-	pan_ws(vsb, htc->ws);
+	VSB_cat(vsb, "doclose = ");
+	pan_stream_close(vsb, htc->doclose);
+	VSB_cat(vsb, "\n");
+	WS_Panic(vsb, htc->ws);
 	VSB_printf(vsb, "{rxbuf_b, rxbuf_e} = {%p, %p},\n",
 	    htc->rxbuf_b, htc->rxbuf_e);
 	VSB_printf(vsb, "{pipeline_b, pipeline_e} = {%p, %p},\n",
@@ -197,13 +177,13 @@ pan_htc(struct vsb *vsb, const struct http_conn *htc)
 	VSB_printf(vsb, "content_length = %jd,\n",
 	    (intmax_t)htc->content_length);
 	VSB_printf(vsb, "body_status = %s,\n",
-	    body_status_2str(htc->body_status));
+	    htc->body_status ? htc->body_status->name : "NULL");
 	VSB_printf(vsb, "first_byte_timeout = %f,\n",
 	    htc->first_byte_timeout);
 	VSB_printf(vsb, "between_bytes_timeout = %f,\n",
 	    htc->between_bytes_timeout);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -213,13 +193,10 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 {
 	int i;
 
-	VSB_printf(vsb, "http[%s] = %p {\n", id, h);
-	if (PAN_already(vsb, h))
+	if (PAN_dump_struct(vsb, h, HTTP_MAGIC, "http[%s]", id))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, h, HTTP_MAGIC);
-	pan_ws(vsb, h->ws);
-	VSB_printf(vsb, "hdrs {\n");
+	WS_Panic(vsb, h->ws);
+	VSB_cat(vsb, "hdrs {\n");
 	VSB_indent(vsb, 2);
 	for (i = 0; i < h->nhd; ++i) {
 		if (h->hd[i].b == NULL && h->hd[i].e == NULL)
@@ -228,26 +205,24 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 		    (int)(h->hd[i].e - h->hd[i].b), h->hd[i].b);
 	}
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
+
 static void
 pan_boc(struct vsb *vsb, const struct boc *boc)
 {
-	VSB_printf(vsb, "boc = %p {\n", boc);
-	if (PAN_already(vsb, boc))
+	if (PAN_dump_struct(vsb, boc, BOC_MAGIC, "boc"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, boc, BOC_MAGIC);
 	VSB_printf(vsb, "refcnt = %u,\n", boc->refcount);
 	VSB_printf(vsb, "state = %s,\n", boc_state_2str(boc->state));
 	VSB_printf(vsb, "vary = %p,\n", boc->vary);
 	VSB_printf(vsb, "stevedore_priv = %p,\n", boc->stevedore_priv);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -257,27 +232,24 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 {
 	const char *p;
 
-	VSB_printf(vsb, "objcore[%s] = %p {\n", typ, oc);
-	if (PAN_already(vsb, oc))
+	if (PAN_dump_struct(vsb, oc, OBJCORE_MAGIC, "objcore[%s]", typ))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, oc, OBJCORE_MAGIC);
 	VSB_printf(vsb, "refcnt = %d,\n", oc->refcnt);
-	VSB_printf(vsb, "flags = {");
+	VSB_cat(vsb, "flags = {");
 
 /*lint -save -esym(438,p) -esym(838,p) -e539 */
 	p = "";
 #define OC_FLAG(U, l, v) \
 	if (oc->flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/oc_flags.h"
-	VSB_printf(vsb, "},\n");
-	VSB_printf(vsb, "exp_flags = {");
+	VSB_cat(vsb, "},\n");
+	VSB_cat(vsb, "exp_flags = {");
 	p = "";
 #define OC_EXP_FLAG(U, l, v) \
 	if (oc->exp_flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/oc_exp_flags.h"
 /*lint -restore */
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 
 	if (oc->boc != NULL)
 		pan_boc(vsb, oc->boc);
@@ -289,18 +261,18 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 		VSB_printf(vsb, " (%s", oc->stobj->stevedore->name);
 		if (strlen(oc->stobj->stevedore->ident))
 			VSB_printf(vsb, " %s", oc->stobj->stevedore->ident);
-		VSB_printf(vsb, ")");
+		VSB_cat(vsb, ")");
 		if (oc->stobj->stevedore->panic) {
-			VSB_printf(vsb, " {\n");
+			VSB_cat(vsb, " {\n");
 			VSB_indent(vsb, 2);
 			oc->stobj->stevedore->panic(vsb, oc);
 			VSB_indent(vsb, -2);
-			VSB_printf(vsb, "}");
+			VSB_cat(vsb, "}");
 		}
 	}
-	VSB_printf(vsb, ",\n");
+	VSB_cat(vsb, ",\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -312,21 +284,18 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 	unsigned m, u;
 	const char *p;
 
-	VSB_printf(vsb, "worker = %p {\n", wrk);
-	if (PAN_already(vsb, wrk))
+	if (PAN_dump_struct(vsb, wrk, WORKER_MAGIC, "worker"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, wrk, WORKER_MAGIC);
-	pan_ws(vsb, wrk->aws);
+	WS_Panic(vsb, wrk->aws);
 
 	m = wrk->cur_method;
-	VSB_printf(vsb, "VCL::method = ");
+	VSB_cat(vsb, "VCL::method = ");
 	if (m == 0) {
-		VSB_printf(vsb, "none,\n");
+		VSB_cat(vsb, "none,\n");
 		return;
 	}
 	if (!(m & 1))
-		VSB_printf(vsb, "inside ");
+		VSB_cat(vsb, "inside ");
 	m &= ~1;
 	hand = VCL_Method_Name(m);
 	if (hand != NULL)
@@ -334,12 +303,7 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 	else
 		VSB_printf(vsb, "0x%x,\n", m);
 
-	hand = VCL_Return_Name(wrk->handling);
-	if (hand != NULL)
-		VSB_printf(vsb, "VCL::return = %s,\n", hand);
-	else
-		VSB_printf(vsb, "VCL::return = 0x%x,\n", wrk->handling);
-	VSB_printf(vsb, "VCL::methods = {");
+	VSB_cat(vsb, "VCL::methods = {");
 	m = wrk->seen_methods;
 	p = "";
 	for (u = 1; m ; u <<= 1) {
@@ -349,9 +313,9 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 			p = ", ";
 		}
 	}
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 static void
@@ -359,8 +323,8 @@ pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 {
 	struct vfp_entry *vfe;
 
-	VSB_printf(vsb, "vfc = %p {\n", vfc);
-	VSB_indent(vsb, 2);
+	if (PAN_dump_struct(vsb, vfc, VFP_CTX_MAGIC, "vfc"))
+		return;
 	VSB_printf(vsb, "failed = %d,\n", vfc->failed);
 	VSB_printf(vsb, "req = %p,\n", vfc->req);
 	VSB_printf(vsb, "resp = %p,\n", vfc->resp);
@@ -368,72 +332,113 @@ pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 	VSB_printf(vsb, "oc = %p,\n", vfc->oc);
 
 	if (!VTAILQ_EMPTY(&vfc->vfp)) {
-		VSB_printf(vsb, "filters = {\n");
+		VSB_cat(vsb, "filters = {\n");
 		VSB_indent(vsb, 2);
 		VTAILQ_FOREACH(vfe, &vfc->vfp, list) {
-			VSB_printf(vsb, "%s = %p {\n",
-				   vfe->vfp->name, vfe);
+			VSB_printf(vsb, "%s = %p {\n", vfe->vfp->name, vfe);
 			VSB_indent(vsb, 2);
 			VSB_printf(vsb, "priv1 = %p,\n", vfe->priv1);
 			VSB_printf(vsb, "priv2 = %zd,\n", vfe->priv2);
 			VSB_printf(vsb, "closed = %d\n", vfe->closed);
 			VSB_indent(vsb, -2);
-			VSB_printf(vsb, "},\n");
+			VSB_cat(vsb, "},\n");
 		}
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "},\n");
+		VSB_cat(vsb, "},\n");
 	}
 
 	VSB_printf(vsb, "obj_flags = 0x%x,\n", vfc->obj_flags);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
-
 static void
 pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 {
 	const char *p;
+	const struct worker *wrk;
 
-	VSB_printf(vsb, "busyobj = %p {\n", bo);
-	if (PAN_already(vsb, bo))
+	if (PAN_dump_struct(vsb, bo, BUSYOBJ_MAGIC, "busyobj"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, bo, BUSYOBJ_MAGIC);
-	pan_ws(vsb, bo->ws);
-	VSB_printf(vsb, "retries = %d, ", bo->retries);
-	VSB_printf(vsb, "flags = {");
-	p = "";
-/*lint -save -esym(438,p) -e539 */
-#define BO_FLAG(l, r, w, d) \
-	if (bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
-#include "tbl/bo_flags.h"
-/*lint -restore */
-	VSB_printf(vsb, "},\n");
+	VSB_printf(vsb, "end = %p,\n", bo->end);
+	VSB_printf(vsb, "retries = %u,\n", bo->retries);
 
-	if (VALID_OBJ(bo->htc, HTTP_CONN_MAGIC))
-		pan_htc(vsb, bo->htc);
+	if (bo->req != NULL)
+		pan_req(vsb, bo->req);
+	if (bo->sp != NULL)
+		pan_sess(vsb, bo->sp);
+	wrk = bo->wrk;
+	if (wrk != NULL)
+		pan_wrk(vsb, wrk);
 
-	if (bo->vfc)
+	if (bo->vfc != NULL)
 		pan_vfp(vsb, bo->vfc);
+	if (bo->vfp_filter_list != NULL) {
+		VSB_printf(vsb, "vfp_filter_list = \"%s\",\n",
+		    bo->vfp_filter_list);
+	}
+	if (bo->vdp_filter_list != NULL) {
+		VSB_printf(vsb, "vdp_filter_list = \"%s\",\n",
+		    bo->vdp_filter_list);
+	}
 
-	VDI_Panic(bo->director_req, vsb, "director_req");
-	if (bo->director_resp == bo->director_req)
-		VSB_printf(vsb, "director_resp = director_req,\n");
-	else
-		VDI_Panic(bo->director_resp, vsb, "director_resp");
+	WS_Panic(vsb, bo->ws);
+	VSB_printf(vsb, "ws_bo = %p,\n", (void *)bo->ws_bo);
+
+	// bereq0 left out
 	if (bo->bereq != NULL && bo->bereq->ws != NULL)
 		pan_http(vsb, "bereq", bo->bereq);
 	if (bo->beresp != NULL && bo->beresp->ws != NULL)
 		pan_http(vsb, "beresp", bo->beresp);
+	if (bo->stale_oc)
+		pan_objcore(vsb, "stale_oc", bo->stale_oc);
 	if (bo->fetch_objcore)
 		pan_objcore(vsb, "fetch", bo->fetch_objcore);
-	if (bo->stale_oc)
-		pan_objcore(vsb, "ims", bo->stale_oc);
-	VCL_Panic(vsb, bo->vcl);
-	VMOD_Panic(vsb);
+
+	if (VALID_OBJ(bo->htc, HTTP_CONN_MAGIC))
+		pan_htc(vsb, bo->htc);
+
+	// fetch_task left out
+
+	VSB_cat(vsb, "flags = {");
+	p = "";
+/*lint -save -esym(438,p) -e539 */
+#define BERESP_FLAG(l, r, w, f, d)				\
+	if (bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
+#define BEREQ_FLAG(l, r, w, d) BERESP_FLAG(l, r, w, 0, d)
+#include "tbl/bereq_flags.h"
+#include "tbl/beresp_flags.h"
+/*lint -restore */
+	VSB_cat(vsb, "},\n");
+
+	// timeouts/timers/acct/storage left out
+
+	pan_storage(vsb, "storage", bo->storage);
+	VDI_Panic(bo->director_req, vsb, "director_req");
+	if (bo->director_resp == bo->director_req)
+		VSB_cat(vsb, "director_resp = director_req,\n");
+	else
+		VDI_Panic(bo->director_resp, vsb, "director_resp");
+	VCL_Panic(vsb, "vcl", bo->vcl);
+	if (wrk != NULL)
+		VPI_Panic(vsb, wrk->vpi, bo->vcl);
+
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
+}
+
+/*--------------------------------------------------------------------*/
+
+static void
+pan_top(struct vsb *vsb, const struct reqtop *top)
+{
+	if (PAN_dump_struct(vsb, top, REQTOP_MAGIC, "top"))
+		return;
+	pan_req(vsb, top->topreq);
+	pan_privs(vsb, top->privs);
+	VCL_Panic(vsb, "vcl0", top->vcl0);
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -441,83 +446,113 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 static void
 pan_req(struct vsb *vsb, const struct req *req)
 {
-	const char *stp;
 	const struct transport *xp;
+	const struct worker *wrk;
 
-	VSB_printf(vsb, "req = %p {\n", req);
-	if (PAN_already(vsb, req))
+	if (PAN_dump_struct(vsb, req, REQ_MAGIC, "req"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, req, REQ_MAGIC);
 	xp = req->transport;
-	VSB_printf(vsb, "vxid = %u, transport = %s", VXID(req->vsl->wid),
+	VSB_printf(vsb, "vxid = %ju, transport = %s", VXID(req->vsl->wid),
 	    xp == NULL ? "NULL" : xp->name);
 
 	if (xp != NULL && xp->req_panic != NULL) {
-		VSB_printf(vsb, " {\n");
+		VSB_cat(vsb, " {\n");
 		VSB_indent(vsb, 2);
 		xp->req_panic(vsb, req);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "}");
+		VSB_cat(vsb, "}");
 	}
-	VSB_printf(vsb, "\n");
-	switch (req->req_step) {
-#define REQ_STEP(l, u, arg) case R_STP_##u: stp = "R_STP_" #u; break;
-#include "tbl/steps.h"
-		default: stp = NULL;
-	}
-	if (stp != NULL)
-		VSB_printf(vsb, "step = %s,\n", stp);
+	VSB_cat(vsb, "\n");
+	if (req->req_step == NULL)
+		VSB_cat(vsb, "step = R_STP_TRANSPORT\n");
 	else
-		VSB_printf(vsb, "step = 0x%x,\n", req->req_step);
+		VSB_printf(vsb, "step = %s\n", req->req_step->name);
+
+	if (req->vfp_filter_list != NULL) {
+		VSB_printf(vsb, "vfp_filter_list = \"%s\",\n",
+		    req->vfp_filter_list);
+	}
+	if (req->vdp_filter_list != NULL) {
+		VSB_printf(vsb, "vdp_filter_list = \"%s\",\n",
+		    req->vdp_filter_list);
+	}
 
 	VSB_printf(vsb, "req_body = %s,\n",
-	    reqbody_status_2str(req->req_body_status));
+	    req->req_body_status ? req->req_body_status->name : "NULL");
 
 	if (req->err_code)
 		VSB_printf(vsb,
 		    "err_code = %d, err_reason = %s,\n", req->err_code,
 		    req->err_reason ? req->err_reason : "(null)");
 
-	VSB_printf(vsb, "restarts = %d, esi_level = %d,\n",
+	VSB_printf(vsb, "restarts = %u, esi_level = %u,\n",
 	    req->restarts, req->esi_level);
 
+	VSB_printf(vsb, "vary_b = %p, vary_e = %p,\n",
+	    req->vary_b, req->vary_e);
+
+	VSB_printf(vsb, "d_ttl = %f, d_grace = %f,\n",
+	    req->d_ttl, req->d_grace);
+
+	pan_storage(vsb, "storage", req->storage);
+
+	VDI_Panic(req->director_hint, vsb, "director_hint");
+
 	if (req->sp != NULL)
 		pan_sess(vsb, req->sp);
 
-	if (req->wrk != NULL)
-		pan_wrk(vsb, req->wrk);
+	wrk = req->wrk;
+	if (wrk != NULL)
+		pan_wrk(vsb, wrk);
 
-	pan_ws(vsb, req->ws);
+	WS_Panic(vsb, req->ws);
 	if (VALID_OBJ(req->htc, HTTP_CONN_MAGIC))
 		pan_htc(vsb, req->htc);
 	pan_http(vsb, "req", req->http);
-	if (req->resp->ws != NULL)
+	if (req->resp != NULL && req->resp->ws != NULL)
 		pan_http(vsb, "resp", req->resp);
+	if (req->vdc != NULL)
+		VDP_Panic(vsb, req->vdc);
 
-	VCL_Panic(vsb, req->vcl);
-	VMOD_Panic(vsb);
+	VCL_Panic(vsb, "vcl", req->vcl);
+	if (wrk != NULL)
+		VPI_Panic(vsb, wrk->vpi, req->vcl);
 
 	if (req->body_oc != NULL)
 		pan_objcore(vsb, "BODY", req->body_oc);
 	if (req->objcore != NULL)
 		pan_objcore(vsb, "REQ", req->objcore);
 
-	VSB_printf(vsb, "flags = {\n");
+	VSB_cat(vsb, "flags = {\n");
 	VSB_indent(vsb, 2);
 #define REQ_FLAG(l, r, w, d) if (req->l) VSB_printf(vsb, #l ",\n");
 #include "tbl/req_flags.h"
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 
 	pan_privs(vsb, req->privs);
 
+	if (req->top != NULL)
+		pan_top(vsb, req->top);
+
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
 
+#define pan_addr(vsb, sp, field) do {					\
+		struct suckaddr *sa;					\
+		char h[VTCP_ADDRBUFSIZE];				\
+		char p[VTCP_PORTBUFSIZE];				\
+									\
+		(void) SES_Get_##field##_addr((sp), &sa);		\
+		if (! VSA_Sane(sa))					\
+			break;						\
+		VTCP_name(sa, h, sizeof h, p, sizeof p);		\
+		VSB_printf((vsb), "%s.ip = %s:%s,\n", #field, h, p);	\
+	} while (0)
+
 static void
 pan_sess(struct vsb *vsb, const struct sess *sp)
 {
@@ -525,27 +560,33 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	const char *cp;
 	const struct transport *xp;
 
-	VSB_printf(vsb, "sp = %p {\n", sp);
-	if (PAN_already(vsb, sp))
+	if (PAN_dump_struct(vsb, sp, SESS_MAGIC, "sess"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, sp, SESS_MAGIC);
-	xp = XPORT_ByNumber(sp->sattr[SA_TRANSPORT]);
-	VSB_printf(vsb, "fd = %d, vxid = %u,\n",
+	VSB_printf(vsb, "fd = %d, vxid = %ju,\n",
 	    sp->fd, VXID(sp->vxid));
 	VSB_printf(vsb, "t_open = %f,\n", sp->t_open);
 	VSB_printf(vsb, "t_idle = %f,\n", sp->t_idle);
-	pan_ws(vsb, sp->ws);
+
+	if (! VALID_OBJ(sp, SESS_MAGIC)) {
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
+		return;
+	}
+
+	WS_Panic(vsb, sp->ws);
+	xp = XPORT_ByNumber(sp->sattr[SA_TRANSPORT]);
 	VSB_printf(vsb, "transport = %s",
 	    xp == NULL ? "<none>" : xp->name);
 	if (xp != NULL && xp->sess_panic != NULL) {
-		VSB_printf(vsb, " {\n");
+		VSB_cat(vsb, " {\n");
 		VSB_indent(vsb, 2);
 		xp->sess_panic(vsb, sp);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "}");
+		VSB_cat(vsb, "}");
 	}
-	VSB_printf(vsb, "\n");
+	VSB_cat(vsb, "\n");
+
+	// duplicated below, remove ?
 	ci = SES_Get_String_Attr(sp, SA_CLIENT_IP);
 	cp = SES_Get_String_Attr(sp, SA_CLIENT_PORT);
 	if (VALID_OBJ(sp->listen_sock, LISTEN_SOCK_MAGIC))
@@ -554,53 +595,76 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	else
 		VSB_printf(vsb, "client = %s %s <unknown>\n", ci, cp);
 
+	if (VALID_OBJ(sp->listen_sock, LISTEN_SOCK_MAGIC)) {
+		VSB_printf(vsb, "local.endpoint = %s,\n",
+			   sp->listen_sock->endpoint);
+		VSB_printf(vsb, "local.socket = %s,\n",
+			   sp->listen_sock->name);
+	}
+	pan_addr(vsb, sp, local);
+	pan_addr(vsb, sp, remote);
+	pan_addr(vsb, sp, server);
+	pan_addr(vsb, sp, client);
+
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
 
-#define BACKTRACE_LEVELS	10
-
 static void
 pan_backtrace(struct vsb *vsb)
 {
-	void *array[BACKTRACE_LEVELS];
-	size_t size;
-	size_t i;
-	char **strings;
-	char *p;
-	char buf[32];
 
-	size = backtrace (array, BACKTRACE_LEVELS);
-	if (size > BACKTRACE_LEVELS) {
-		VSB_printf(vsb, "Backtrace not available (ret=%zu)\n", size);
-		return;
-	}
-	VSB_printf(vsb, "Backtrace:\n");
+	VSB_cat(vsb, "Backtrace:\n");
 	VSB_indent(vsb, 2);
-	for (i = 0; i < size; i++) {
-		bprintf(buf, "%p", array[i]);
-		VSB_printf(vsb, "%s: ", buf);
-		strings = backtrace_symbols(&array[i], 1);
-		if (strings == NULL || strings[0] == NULL) {
-			VSB_printf(vsb, "(?)");
-		} else {
-			p = strings[0];
-			if (!memcmp(buf, p, strlen(buf))) {
-				p += strlen(buf);
-				if (*p == ':')
-					p++;
-				while (*p == ' ')
-					p++;
-			}
-			VSB_printf(vsb, "%s", p);
-		}
-		VSB_printf (vsb, "\n");
-	}
+	VBT_format(vsb);
 	VSB_indent(vsb, -2);
 }
 
+#ifdef HAVE_PTHREAD_GETATTR_NP
+static void
+pan_threadattr(struct vsb *vsb)
+{
+	pthread_attr_t attr[1];
+	size_t sz;
+	void *addr;
+
+	if (pthread_getattr_np(pthread_self(), attr) != 0)
+		return;
+
+	VSB_cat(vsb, "pthread.attr = {\n");
+	VSB_indent(vsb, 2);
+
+	if (pthread_attr_getguardsize(attr, &sz) == 0)
+		VSB_printf(vsb, "guard = %zu,\n", sz);
+	if (pthread_attr_getstack(attr, &addr, &sz) == 0) {
+		VSB_printf(vsb, "stack_bottom = %p,\n", addr);
+		VSB_printf(vsb, "stack_top = %p,\n", (char *)addr + sz);
+		VSB_printf(vsb, "stack_size = %zu,\n", sz);
+	}
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "}\n");
+	(void) pthread_attr_destroy(attr);
+}
+#endif
+
+static void
+pan_argv(struct vsb *vsb)
+{
+	int i;
+
+	VSB_cat(pan_vsb, "argv = {\n");
+	VSB_indent(vsb, 2);
+	for (i = 0; i < heritage.argc; i++) {
+		VSB_printf(vsb, "[%d] = ", i);
+		VSB_quote(vsb, heritage.argv[i], -1, VSB_QUOTE_CSTR);
+		VSB_cat(vsb, ",\n");
+	}
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "}\n");
+
+}
 /*--------------------------------------------------------------------*/
 
 static void __attribute__((__noreturn__))
@@ -610,12 +674,25 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	const char *q;
 	struct req *req;
 	struct busyobj *bo;
+	struct worker *wrk;
 	struct sigaction sa;
-	int err = errno;
+	int i, err = errno;
 
-	AZ(pthread_mutex_lock(&panicstr_mtx)); /* Won't be released,
-						  we're going to die
-						  anyway */
+	if (pthread_getspecific(panic_key) != NULL) {
+		VSB_cat(pan_vsb, "\n\nPANIC REENTRANCY\n\n");
+		abort();
+	}
+
+	/* If we already panicking in another thread, do nothing */
+	do {
+		i = pthread_mutex_trylock(&panicstr_mtx);
+		if (i != 0)
+			sleep (1);
+	} while (i != 0);
+
+	assert (VSB_len(pan_vsb) == 0);
+
+	AZ(pthread_setspecific(panic_key, pan_vsb));
 
 	/*
 	 * should we trigger a SIGSEGV while handling a panic, our sigsegv
@@ -640,7 +717,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		break;
 	case VAS_MISSING:
 		VSB_printf(pan_vsb,
-		    "Missing errorhandling code in %s(), %s line %d:\n"
+		    "Missing error handling code in %s(), %s line %d:\n"
 		    "  Condition(%s) not true.\n",
 		    func, file, line, cond);
 		break;
@@ -649,8 +726,8 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		    "Incomplete code in %s(), %s line %d:\n",
 		    func, file, line);
 		break;
-	default:
 	case VAS_ASSERT:
+	default:
 		VSB_printf(pan_vsb,
 		    "Assert error in %s(), %s line %d:\n"
 		    "  Condition(%s) not true.\n",
@@ -658,7 +735,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		break;
 	}
 	VSB_printf(pan_vsb, "version = %s, vrt api = %u.%u\n",
-	    VCS_version, VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
+	    VCS_String("V"), VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
 	VSB_printf(pan_vsb, "ident = %s,%s\n",
 	    heritage.ident, Waiter_GetName());
 	VSB_printf(pan_vsb, "now = %f (mono), %f (real)\n",
@@ -667,11 +744,19 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	pan_backtrace(pan_vsb);
 
 	if (err)
-		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, strerror(err));
+		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, VAS_errtxt(err));
+
+	pan_argv(pan_vsb);
+
+	VSB_printf(pan_vsb, "pthread.self = %p\n", TRUST_ME(pthread_self()));
 
 	q = THR_GetName();
 	if (q != NULL)
-		VSB_printf(pan_vsb, "thread = (%s)\n", q);
+		VSB_printf(pan_vsb, "pthread.name = (%s)\n", q);
+
+#ifdef HAVE_PTHREAD_GETATTR_NP
+	pan_threadattr(pan_vsb);
+#endif
 
 	if (!FEATURE(FEATURE_SHORT_PANIC)) {
 		req = THR_GetRequest();
@@ -684,22 +769,33 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		pan_busyobj(pan_vsb, bo);
 		if (bo != NULL)
 			VSL_Flush(bo->vsl, 0);
+		wrk = THR_GetWorker();
+		VSB_cat(pan_vsb, "thr.");
+		pan_wrk(pan_vsb, wrk);
+		VMOD_Panic(pan_vsb);
+		pan_pool(pan_vsb);
 	} else {
-		VSB_printf(pan_vsb,
-		    "Feature short panic supressed details.\n");
+		VSB_cat(pan_vsb, "Feature short panic suppressed details.\n");
 	}
-	VSB_printf(pan_vsb, "\n");
+	VSB_cat(pan_vsb, "\n");
 	VSB_putc(pan_vsb, '\0');	/* NUL termination */
 
-	if (FEATURE(FEATURE_NO_COREDUMP))
-		exit(4);
-	else
+	v_gcov_flush();
+
+	/*
+	 * Do a little song and dance for static checkers which
+	 * are not smart enough to figure out that calling abort()
+	 * with a mutex held is OK and probably very intentional.
+	 */
+	if (pthread_getspecific(panic_key))	/* ie: always */
 		abort();
+	PTOK(pthread_mutex_unlock(&panicstr_mtx));
+	abort();
 }
 
 /*--------------------------------------------------------------------*/
 
-static void v_matchproto_(cli_func_t)
+static void v_noreturn_ v_matchproto_(cli_func_t)
 ccf_panic(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -707,6 +803,8 @@ ccf_panic(struct cli *cli, const char * const *av, void *priv)
 	(void)av;
 	AZ(priv);
 	AZ(strcmp("", "You asked for it"));
+	/* NOTREACHED */
+	abort();
 }
 
 /*--------------------------------------------------------------------*/
@@ -722,14 +820,13 @@ void
 PAN_Init(void)
 {
 
-	AZ(pthread_mutex_init(&panicstr_mtx, NULL));
+	PTOK(pthread_mutex_init(&panicstr_mtx, &mtxattr_errorcheck));
 	VAS_Fail_Func = pan_ic;
 	pan_vsb = &pan_vsb_storage;
 	AN(heritage.panic_str);
 	AN(heritage.panic_str_len);
-	AN(VSB_new(pan_vsb, heritage.panic_str, heritage.panic_str_len,
-	    VSB_FIXEDLEN));
-	VSB_printf(pan_vsb, "This is a test\n");
+	AN(VSB_init(pan_vsb, heritage.panic_str, heritage.panic_str_len));
+	VSB_cat(pan_vsb, "This is a test\n");
 	AZ(VSB_finish(pan_vsb));
 	VSB_clear(pan_vsb);
 	heritage.panic_str[0] = '\0';
