@@ -47,6 +47,8 @@
 #include <sys/socket.h>
 
 #include "mgt/mgt.h"
+#include "acceptor/cache_acceptor.h"
+#include "acceptor/mgt_acceptor.h"
 #include "common/heritage.h"
 
 #include "hash/hash_slinger.h"
@@ -90,6 +92,8 @@ mgt_DumpOptions(void)
 	       "compiled VCLs etc.\n\n");
 	printf("  In performance critical applications, this directory "
 	       "should be on a RAM backed filesystem.\n\n");
+	printf("  When running multiple varnishd instances, separate "
+	       "directories need to be used.\n\n");
 	VIN_DumpDefaults();
 }
 
@@ -445,9 +449,8 @@ static int v_matchproto_(vev_cb_f)
 mgt_sigint(const struct vev *e, int what)
 {
 
-	(void)e;
 	(void)what;
-	MGT_Complain(C_ERR, "Manager got %s", e->name);
+	MGT_Complain(C_ERR, "Manager got %s from PID %jd", e->name, (intmax_t)e->siginfo->si_pid);
 	(void)fflush(stdout);
 	if (MCH_Running())
 		MCH_Stop_Child();
@@ -564,7 +567,7 @@ mgt_process_f_arg(struct cli *cli, unsigned C_flag, void **fap)
 	return (retval);
 }
 
-static const char *
+static char *
 create_bogo_n_arg(void)
 {
 	struct vsb *vsb;
@@ -637,6 +640,7 @@ main(int argc, char * const *argv)
 	pid_t pid;
 	struct arg_list *alp;
 	int first_arg = 1;
+	struct vsb *vsb;
 
 	if (argc == 2 && !strcmp(argv[1], "--optstring")) {
 		printf("%s\n", opt_spec);
@@ -775,6 +779,7 @@ main(int argc, char * const *argv)
 
 	/* Various initializations */
 	VTAILQ_INIT(&heritage.socks);
+	VCA_Config();
 	mgt_evb = VEV_New();
 	AN(mgt_evb);
 
@@ -793,7 +798,7 @@ main(int argc, char * const *argv)
 	VTAILQ_FOREACH(alp, &arglist, list) {
 		switch(alp->arg[0]) {
 		case 'a':
-			MAC_Arg(alp->val);
+			VCA_Arg(alp->val);
 			break;
 		case 'f':
 			if (*alp->val != '\0')
@@ -838,8 +843,11 @@ main(int argc, char * const *argv)
 
 	assert(d_flag == 0 || F_flag == 0);
 
-	if (C_flag && n_arg == NULL)
-		n_arg = create_bogo_n_arg();
+	p = NULL;
+	if (C_flag && n_arg == NULL) {
+		p = create_bogo_n_arg();
+		n_arg = p;
+	}
 
 	if (S_arg != NULL && !strcmp(S_arg, "none")) {
 		fprintf(stderr,
@@ -856,6 +864,8 @@ main(int argc, char * const *argv)
 
 	workdir = VIN_n_Arg(n_arg);
 	AN(workdir);
+	if (p != NULL)
+		free(p);
 
 	if (i_arg == NULL || *i_arg == '\0')
 		i_arg = mgt_HostName();
@@ -869,9 +879,12 @@ main(int argc, char * const *argv)
 
 	openlog("varnishd", LOG_PID, LOG_LOCAL0);
 
-	if (VJ_make_workdir(workdir))
-		ARGV_ERR("Cannot create working directory (%s): %s\n",
-		    workdir, VAS_errtxt(errno));
+	vsb = VSB_new_auto();
+	AN(vsb);
+	o = VJ_make_workdir(workdir, vsb);
+	MGT_ComplainVSB(o ? C_ERR : C_INFO, vsb);
+	if (o)
+		ARGV_EXIT;
 
 	VJ_master(JAIL_MASTER_SYSTEM);
 #ifdef RLIMIT_MEMLOCK
@@ -888,25 +901,16 @@ main(int argc, char * const *argv)
 	AZ(system("rm -rf vmod_cache vext_cache worker_tmpdir"));
 	VJ_master(JAIL_MASTER_LOW);
 
-	if (VJ_make_subdir("vmod_cache", "VMOD cache", NULL)) {
-		ARGV_ERR(
-		    "Cannot create vmod directory (%s/vmod_cache): %s\n",
-		    workdir, VAS_errtxt(errno));
-	}
-
-	if (arg_list_count("E") &&
-	    VJ_make_subdir("vext_cache", "VEXT cache", NULL)) {
-		ARGV_ERR(
-		    "Cannot create vmod directory (%s/vext_cache): %s\n",
-		    workdir, VAS_errtxt(errno));
-	}
-
-	if (VJ_make_subdir("worker_tmpdir",
-	    "TMPDIR for the worker process", NULL)) {
-		ARGV_ERR(
-		    "Cannot create vmod directory (%s/worker_tmpdir): %s\n",
-		    workdir, VAS_errtxt(errno));
-	}
+	VSB_clear(vsb);
+	o = VJ_make_subdir("vmod_cache", "VMOD cache", vsb) ||
+	    VJ_make_subdir("worker_tmpdir",
+		"TMPDIR for the worker process", vsb) ||
+	    (arg_list_count("E") &&
+		VJ_make_subdir("vext_cache", "VEXT cache", vsb));
+	MGT_ComplainVSB(o ? C_ERR : C_INFO, vsb);
+	VSB_destroy(&vsb);
+	if (o)
+		ARGV_EXIT;
 
 	o = open("worker_tmpdir", O_RDONLY);
 	VJ_master(JAIL_MASTER_SYSTEM);
@@ -946,7 +950,7 @@ main(int argc, char * const *argv)
 	alp->priv = create_pid_file(&pid, "%s/_.pid", workdir);
 
 	if (VTAILQ_EMPTY(&heritage.socks))
-		MAC_Arg(":80\0");	// XXX: extra NUL for FlexeLint
+		VCA_Arg(":80\0");	// XXX: extra NUL for FlexeLint
 
 	assert(!VTAILQ_EMPTY(&heritage.socks));
 
